#!/usr/bin/env python
# Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
# Copyright (C) 2009-2017 German Aerospace Center (DLR) and others.
# This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v2.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v20.html

# @file    runner.py
# @author  Lena Kalleske
# @author  Daniel Krajzewicz
# @author  Michael Behrisch
# @author  Jakob Erdmann
# @date    2009-03-26
# @version $Id$

'''
netconvert  -i all_red.xml --node-files cross.nod.xml --edge-files cross.edg.xml -t cross.con.xml  -o cross.net.
'''

from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import optparse

# we need to import python modules from the $SUMO_HOME/tools directory
try:
    sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
    from sumolib import checkBinary  # noqa
except ImportError:
    sys.exit(
        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")

import traci

def run():

    while traci.simulation.getMinExpectedNumber() > 0:
        traci.simulationStep()

    traci.close()
    sys.stdout.flush()


def get_options():
    optParser = optparse.OptionParser()
    optParser.add_option("--nogui", action="store_true",
                         default=False, help="run the commandline version of sumo")

    optParser.add_option("--generate_detectors", action="store_true",
                         default=False, help="generate detector files for this network")

    options, args = optParser.parse_args()
    return options

def map_lane_to_phase(tl):

    # Get controlled lanes.
    controlled_lanes = set(traci.trafficlight.getControlledLanes(tl))

    print('traffic light : ', tl, ' controls lanes: ', controlled_lanes)

    # controlled_links[0] should contain the incoming links.
    # this list should be sorted by the signal index.
    # controlled_links = set(controlled_links[0])
    controlled_links = traci.trafficlight.getControlledLinks(tl)
    print('traffic light : ', tl, ' controls links:')
    for elem in controlled_links:
        print(elem)


# this is the main entry point of this script
if __name__ == "__main__":

    # Get options
    options = get_options()

    #############################################
    ## Generate detector loops on the network ###
    #############################################
    if options.generate_detectors:
        # Generate detector file
        os.system('python3 generate_detector_file.py -d ' + network_dir)

    # this script has been called from the command line. It will start sumo as a server, then connect and run
    if options.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')

    # this is the normal way of using traci. sumo is started as a subprocess and
    # then the python script connects and runs
    traci.start([sumoBinary,"--waiting-time-memory", "360", "--step-length", "0.5", "-c", "./cross.sumocfg", "--tripinfo-output", "tripinfo.xml"])

    tl_list = traci.trafficlight.getIDList()

    # Map lane to a phase
    map_lane_to_phase(tl_list[0])

    # Run simulation
    run()
